<?php

namespace App\Libs\Actions;

use App\Http\Resources\LevelResource;
use App\Http\Resources\SubjectResource;
use App\Models\Level;
use Illuminate\Http\JsonResponse;
use Illuminate\Http\Resources\Json\AnonymousResourceCollection;

/**
 *
 */
class LevelAction
{
    /**
     * @var Level
     */
    private Level $model;

    /**
     * @param Level $model
     */
    public function __construct(Level $model)
    {
        $this->model = $model;
    }

    /**
     * create subject
     * @param $request
     * @return JsonResponse
     */
    public function createLevelAction($request): JsonResponse
    {
        try {
            $subject = $this->model->create([
                'category_id' => $request->category_id,
                'name' => $request->name,
                'description' => $request->description
            ]);
            return response()->json([
                'message' => 'Level created successfully',
                'data' => new LevelResource($subject),
                'success' => true
            ], 201);
        } catch (\Exception $e) {
            return response()->json([
                'message' => 'Sorry unable to  create level',
                'error' => $e->getMessage(),
                'success' => false
            ], 400);
        }
    }

    /**
     * get single level
     * @param $id
     * @return LevelResource
     */
    public function getSingleLevelAction($id): LevelResource
    {
        $level = $this->model->findOrFail($id);
        return (new LevelResource($level))->additional( [
            'message' => "Level details",
            'success' => true
        ], 200);
    }

    /**
     * get all levels by category id
     * @param $id
     * @return JsonResponse|AnonymousResourceCollection
     */
    public function getAllLevelByCategoryAction($id): JsonResponse|AnonymousResourceCollection
    {
        $levels = $this->model->where('category_id', '=', $id)->latest()->get();
        if (count($levels) < 1) {
            return response()->json([
                'message' => 'Sorry no level found',
                'success' => false
            ], 404);
        }else {
            return LevelResource::collection($levels)->additional([
                'message' => "All levels",
                'success' => true
            ], 200);
        }
    }

    /**
     * get all level
     * @return JsonResponse|AnonymousResourceCollection
     */
    public function getAllLevelAction(): JsonResponse|AnonymousResourceCollection
    {
        $levels = $this->model->latest()->paginate(20);
        if (count($levels) < 1) {
            return response()->json([
                'message' => 'Sorry no level found',
                'success' => false
            ], 404);
        }else {
            return LevelResource::collection($levels)->additional([
                'message' => "All levels",
                'success' => true
            ], 200);
        }
    }

    /**
     * update level
     * @param $request
     * @param $id
     * @return JsonResponse
     */
    public function updateLevelAction($request, $id): JsonResponse
    {
        $subject =  $this->model->findOrFail($id);
        $subject->slug = null;
        try {
            $subject->update([
                'name' => empty($request->name) ? $subject->name : $request->name,
                'description' =>   empty($request->description) ? $subject->description : $request->description
            ]);
            return response()->json([
                'message' => 'Level edited successfully',
                'data' => new LevelResource($subject),
                'success' => true
            ]);
        } catch (\Exception $e) {
            return response()->json([
                'message' => 'Sorry unable to edit category',
                'error' => $e->getMessage(),
                'success' => false
            ], 400);
        }
    }

    /**
     * delete level
     * @param $id
     * @return JsonResponse
     */
    public function deleteLevelAction($id): JsonResponse
    {
        $subject =  $this->model->findOrFail($id);
        try {
            $subject->delete();
            return response()->json([
                'message' => 'Level deleted successfully',
                'data' => new SubjectResource($subject),
                'success' => true
            ], 200);
        }catch (\Exception $e) {
            return response()->json([
                'message' => 'Sorry unable to delete level',
                'error' => $e->getMessage(),
                'success' => false
            ], 400);
        }
    }
}
