<?php

namespace App\Libs\Actions;

use App\Libs\Traits\AuthUserTrait;
use App\Models\Student;
use App\Http\Resources\StudentResource;
use Illuminate\Http\JsonResponse;
use Illuminate\Http\Resources\Json\AnonymousResourceCollection;

/**
 * Student action
 */
class StudentAction
{
    use AuthUserTrait;

    /**
     * @var Student
     */
    private Student $model;

    /**
     * @param Student $model
     */
    public function __construct(Student $model)
    {
        $this->model = $model;
    }

    /**
     * create student
     * @param $request
     * @return JsonResponse
     */
    public function createStudentAccountAction($request): JsonResponse
    {
        try {
            $student = $this->model->create([
                'guardian_id' => $this->getAuthUser()->guardian->id,
                'name' => $request->name,
                'age' => $request->age,
                'level_id' => $request->level_id
            ]);
            return response()->json([
                'message' => 'Student created successfully',
                'data' => new StudentResource($student),
                'success' => true
            ], 201);
        } catch (\Exception $e) {
            return response()->json([
                'message' => 'Sorry unable to  create student',
                'error' => $e->getMessage(),
                'success' => false
            ], 400);
        }
    }

    /**
     * get all student action
     * @return JsonResponse|AnonymousResourceCollection
     */
    public function getAllStudentsAction(): JsonResponse|AnonymousResourceCollection
    {
        $students = $this->getAuthUser()->superadmin ? $this->model->latest()->paginate(20) :
            $this->model->where('guardian_id', '=', $this->getAuthUser()->guardian->id)->latest()->paginate(20);
        if (count($students) < 1) {
            return response()->json([
                'message' => 'Sorry no student found',
                'success' => false
            ], 404);
        }else {
            return StudentResource::collection($students)->additional([
                'message' => "All students",
                'success' => true
            ], 200);
        }
    }

    /**
     * get single student action
     * @param $id
     * @return StudentResource
     */
    public function getSingleStudentAction($id): StudentResource
    {
        $student = $this->model->findOrFail($id);
        return (new StudentResource($student))->additional( [
            'message' => "Category details",
            'success' => true
        ], 200);
    }

    /**
     * update student action
     * @param $request
     * @param $id
     * @return JsonResponse
     */
    public function updateStudentAccountAction($request, $id): JsonResponse
    {
        $student =  $this->model->findOrFail($id);
        $student->slug = null;
        try {
            $student->update([
                'name' => empty($request->name) ? $student->name : $request->name,
                'age' => empty($request->age) ? $student->age : $request->age,
                'level_id' => empty($request->level_id) ? $student->level_id : $request->level_id
            ]);
            return response()->json([
                'message' => 'Student edited successfully',
                'data' => new StudentResource($student),
                'success' => true
            ]);
        } catch (\Exception $e) {
            return response()->json([
                'message' => 'Sorry unable to edit student',
                'error' => $e->getMessage(),
                'success' => false
            ], 400);
        }
    }

    /**
     * delete student action
     * @param $id
     * @return JsonResponse
     */
    public function deleteStudentAction($id): JsonResponse
    {
        $category =  $this->model->findOrFail($id);
        try {
            $category->delete();
            return response()->json([
                'message' => 'Student deleted successfully',
                'data' => new StudentResource($category),
                'success' => true
            ], 200);
        }catch (\Exception $e) {
            return response()->json([
                'message' => 'Sorry unable to delete student',
                'error' => $e->getMessage(),
                'success' => false
            ], 400);
        }
    }


}
