<?php

namespace App\Libs\Actions;

use App\Http\Resources\SubjectResource;
use App\Models\Subject;
use Illuminate\Http\JsonResponse;
use Illuminate\Http\Resources\Json\AnonymousResourceCollection;

class SubjectAction
{
    /**
     * @var Subject
     */
    private Subject $model;

    /**
     * @param Subject $model
     */
    public function __construct(Subject $model)
    {
        $this->model = $model;
    }

    /**
     * create subject
     * @param $request
     * @return JsonResponse
     */
    public function createSubjectAction($request): JsonResponse
    {
        try {
            $subject = $this->model->create([
                'category_id' => $request->category_id,
                'name' => $request->name,
                'description' => $request->description
            ]);
            return response()->json([
                'message' => 'Subject created successfully',
                'data' => new SubjectResource($subject),
                'success' => true
            ], 201);
        } catch (\Exception $e) {
            return response()->json([
                'message' => 'Sorry unable to  create subject',
                'error' => $e->getMessage(),
                'success' => false
            ], 400);
        }
    }

    /**
     * get single subject
     * @param $id
     * @return SubjectResource
     */
    public function getSingleSubjectAction($id): SubjectResource
    {
        $subject = $this->model->with(['category'])->findOrFail($id);
        return (new SubjectResource($subject))->additional( [
            'message' => "Subject details",
            'success' => true
        ], 200);
    }

    /**
     * get all subjects by category id
     * @param $id
     * @return JsonResponse|AnonymousResourceCollection
     */
    public function getAllSubjectByCategoryAction($id): JsonResponse|AnonymousResourceCollection
    {
        $levels = $this->model->where('category_id', '=', $id)->latest()->get();
        if (count($levels) < 1) {
            return response()->json([
                'message' => 'Sorry no subject found',
                'success' => false
            ], 404);
        }else {
            return SubjectResource::collection($levels)->additional([
                'message' => "All subject",
                'success' => true
            ], 200);
        }
    }

    /**
     * get all subjects
     * @return JsonResponse|AnonymousResourceCollection
     */
    public function getAllSubjectAction(): JsonResponse|AnonymousResourceCollection
    {
        $categories = $this->model->latest()->paginate(20);
        if (count($categories) < 1) {
            return response()->json([
                'message' => 'Sorry no subject found',
                'success' => false
            ], 404);
        }else {
            return SubjectResource::collection($categories)->additional([
                'message' => "All subjects",
                'success' => true
            ], 200);
        }
    }

    /**
     * update subject
     * @param $request
     * @param $id
     * @return JsonResponse
     */
    public function updateSubjectAction($request, $id): JsonResponse
    {
        $subject =  $this->model->findOrFail($id);
        $subject->slug = null;
        try {
            $subject->update([
                'name' => empty($request->name) ? $subject->name : $request->name,
                'description' =>   empty($request->description) ? $subject->description : $request->description
            ]);
            return response()->json([
                'message' => 'Subject edited successfully',
                'data' => new SubjectResource($subject),
                'success' => true
            ]);
        } catch (\Exception $e) {
            return response()->json([
                'message' => 'Sorry unable to edit category',
                'error' => $e->getMessage(),
                'success' => false
            ], 400);
        }
    }

    /**
     * delete subject
     * @param $id
     * @return JsonResponse
     */
    public function deleteSubjectAction($id): JsonResponse
    {
        $subject =  $this->model->findOrFail($id);
        try {
            $subject->delete();
            return response()->json([
                'message' => 'Subject deleted successfully',
                'data' => new SubjectResource($subject),
                'success' => true
            ], 200);
        }catch (\Exception $e) {
            return response()->json([
                'message' => 'Sorry unable to delete subject',
                'error' => $e->getMessage(),
                'success' => false
            ], 400);
        }
    }
}
