<?php

namespace App\Libs\Actions;

use App\Http\Resources\TeacherResource;
use App\Http\Resources\UserResource;
use App\Libs\Helpers\QueryHelper;
use App\Libs\Traits\AuthUserTrait;
use App\Models\Subject;
use App\Models\Teacher;
use Illuminate\Http\JsonResponse;
use App\Libs\Helpers\FileProcessing;
use Illuminate\Http\Resources\Json\AnonymousResourceCollection;
use App\Models\Classes;
use App\Models\TeacherSubject;

/**
 * Teacher action
 */
class TeacherAction
{
    use AuthUserTrait;

    /**
     * @var Teacher
     */
    private Teacher $model;
    /**
     * @var QueryHelper
     */
    private QueryHelper $queryHelper;
    /**
     * @var FileProcessing
     */
    private FileProcessing $fileProcessing;
    /**
     * @var Classes
     */
    private Classes $classesModel;
    /**
     * @var TeacherSubject
     */
    private TeacherSubject $teacherSubjectModel;
    /**
     * @var Subject
     */
    private Subject $subjectModel;

    /**
     * @param Teacher $model
     * @param QueryHelper $queryHelper
     * @param FileProcessing $fileProcessing
     * @param Classes $classesModel
     * @param TeacherSubject $teacherSubjectModel
     * @param Subject $subjectModel
     */
    public function __construct(Teacher $model, QueryHelper $queryHelper, FileProcessing $fileProcessing, Classes $classesModel, TeacherSubject $teacherSubjectModel, Subject $subjectModel)
    {
        $this->model = $model;
        $this->queryHelper = $queryHelper;
        $this->fileProcessing = $fileProcessing;
        $this->classesModel = $classesModel;
        $this->teacherSubjectModel = $teacherSubjectModel;
        $this->subjectModel = $subjectModel;
    }

    /**
     * add subjects as a teacher
     * @param $request
     * @return JsonResponse
     */
    public function addSubjectsAction($request): JsonResponse
    {
        try {
            $this->model->where('user_id', '=', $this->getAuthUser()->id)->update([
                'category_id' => $request->category
            ]);
            $this->createClassesForSubject($request->subjects);
            return response()->json([
                'message' => 'Subject added successfully',
                'data' => new UserResource($this->getAuthUser()),
                'success' => true
            ], 200);
        }catch (\Exception $e) {
            return response()->json([
                'message' => 'Sorry unable to add subject',
                'error' => $e->getMessage(),
                'success' => false
            ], 400);
        }
    }

    /**
     * create classes
     * @param array $subjects
     * @return void
     */
    public function createClassesForSubject(array $subjects): void
    {
        foreach ($subjects as $key => $subject) {
            $this->teacherSubjectModel->create([
                'teacher_id' => $this->getAuthUser()->teacher->id,
                "variation" => $subject['variation'],
                "day" => $subject['day'],
                "time" => $subject['time'],
                "end_time" => $subject['end_time'],
                "subject_id" => $subject['subject']
            ]);
            $this->classesModel->create([
                'teacher_id' => $this->getAuthUser()->teacher->id,
                'subject_id' => $subject['subject'],
                'day' => $subject['day'],
                'time' => $subject['time'],
                "end_time" => $subject['end_time'],
                'number_of_students' => 0,
                "variation" => $subject['variation']
            ]);
        }
    }

    /**
     * update educational qualification by teacher
     * @param $request
     * @return JsonResponse
     */
    public function updateEducationalQualificationAction($request): JsonResponse
    {
        try {
            $this->model->where('user_id', '=', $this->getAuthUser()->id)->update([
                'school_attend' => $request->school_attend,
                'degree_attained' => $request->degree_attained,
                'certificate_url' => $this->fileProcessing->pushImageFile($request, 'certificate_url')
            ]);
            return response()->json([
                'message' => 'Teachers qualification updated successfully',
                'data' => new UserResource($this->getAuthUser()),
                'success' => true
            ], 200);
        } catch (\Exception $e) {
            return response()->json([
                'message' => 'Sorry unable to add educational qualification',
                'error' => $e->getMessage(),
                'success' => false
            ], 400);
        }
    }

    /**
     * approve teachers account by admin
     * @param $slug
     * @return JsonResponse
     */
    public function approveTeachersAccountAction($slug): JsonResponse
    {
        $teacher = $this->model->where('slug', '=', $slug)->firstOrFail();
        try {
            $teacher->update([
                'is_approved' => true
            ]);
            return response()->json([
                'message' => 'Teachers account approved successfully',
                'data' => new UserResource($teacher->user),
                'success' => true
            ], 200);
        }catch (\Exception $e) {
            return response()->json([
                'message' => 'Sorry unable to approve teachers account',
                'error' => $e->getMessage(),
                'success' => false
            ], 400);
        }
    }

    /**
     * get teachers by subject
     * @param $request
     * @return JsonResponse|AnonymousResourceCollection
     */
    public function getTeachersBySubjectAction($request): JsonResponse|AnonymousResourceCollection
    {
        $subject =  $this->subjectModel->where('name', '=', $request->subject )->first();
        $subject_id = $subject->id;
        $teachers = $this->model->whereHas('subjects', function ($query) use ($subject_id) {
            $query->where('subject_id', '=', $subject_id);
        })->latest()->paginate(20);
        if (count($teachers) < 1) {
            return response()->json([
                'message' => 'Sorry no teacher found',
                'success' => false
            ], 404);
        }else {
            return TeacherResource::collection($teachers)->additional([
                'message' => "All teachers for $request->subject",
                'success' => true
            ], 200);
        }
    }

    /**
     * get all teachers
     * @return JsonResponse|AnonymousResourceCollection
     */
    public function getAllTeachersAction(): JsonResponse|AnonymousResourceCollection
    {
        $teachers = $this->model->latest()->paginate(20);
        if (count($teachers) < 1) {
            return response()->json([
                'message' => 'Sorry no teacher found',
                'success' => false
            ], 404);
        }else {
            return TeacherResource::collection($teachers)->additional([
                'message' => "All teachers fetched successfully",
                'success' => true
            ], 200);
        }
    }

    /**
     * get single teacher
     * @param $id
     * @return TeacherResource
     */
    public function getSingleTeachersAction($id): TeacherResource
    {
        $teacher = $this->model->findOrFail($id);
        return (new TeacherResource($teacher))->additional( [
            'message' => "Teacher details",
            'success' => true
        ], 200);
    }


}
