<?php

namespace App\Libs\Actions;

use App\Http\Resources\ClassesResource;
use App\Http\Resources\TeacherRequestResource;
use App\Http\Resources\UserResource;
use App\Libs\Helpers\CheckEngagementStatusHelper;
use App\Libs\Helpers\QueryHelper;
use App\Libs\Traits\AuthUserTrait;
use App\Models\TeacherRequest;
use Illuminate\Config\Repository;
use Illuminate\Contracts\Foundation\Application;
use Illuminate\Http\JsonResponse;
use Illuminate\Http\Resources\Json\AnonymousResourceCollection;
use Illuminate\Support\Facades\Http;
use App\Models\Classes;

/**
 * TeacherRequest action
 */
class TeacherRequestAction
{
    use AuthUserTrait;

    /**
     * @var TeacherRequest
     */
    private TeacherRequest $model;
    /**
     * @var QueryHelper
     */
    private QueryHelper $queryHelper;
    /**
     * @var mixed|Repository|Application
     */
    private mixed $apiUrl;
    /**
     * @var mixed|Repository|Application
     */
    private mixed $apiKey;
    private Classes $classesModel;
    private CheckEngagementStatusHelper $checkEngagementStatusHelper;

    /**
     * @param TeacherRequest $model
     * @param QueryHelper $queryHelper
     * @param Classes $classesModel
     */
    public function __construct(TeacherRequest $model, QueryHelper $queryHelper, Classes $classesModel, CheckEngagementStatusHelper $checkEngagementStatusHelper)
    {
        $this->model = $model;
        $this->queryHelper = $queryHelper;
        $this->apiKey = config('virtualclassroom-config.class.api_key');
        $this->apiUrl =  config('virtualclassroom-config.class.api_url');
        $this->classesModel = $classesModel;
        $this->checkEngagementStatusHelper = $checkEngagementStatusHelper;
    }

    /**
     * engage teacher
     * @param $request
     * @return JsonResponse
     */
    public function engageTeacherByStudentAction($request): JsonResponse
    {
        if (!$this->getAuthUser()->guardian->is_subscribed) {
            return response()->json([
                'message' => 'Sorry you are not yet subscribed to a active plan',
                'success' => false
            ], 403);
        } else {
            if ($this->checkEngagementStatusHelper->checkStatusHelper()) {
                try {
                    $this->model->create([
                        'teacher_id' => $request->teacher_id,
                        'student_id' => $request->student_id,
                        'guardian_id' => $this->getAuthUser()->guardian->id,
                        'subject_id' => $request->subject_id
                    ]);
                    return response()->json([
                        'message' => 'Engagement teacher request sent successfully',
                        'data' => new UserResource($this->getAuthUser()),
                        'success' => true
                    ], 201);
                }catch (\Exception $e) {
                    return response()->json([
                        'message' => 'Sorry unable to engagement teacher',
                        'error' => $e->getMessage(),
                        'success' => false
                    ], 400);
                }
            } else {
                return response()->json([
                    'message' => 'Sorry you hav exceeded your plan limit',
                    'success' => false
                ], 403);
            }
        }
    }

    /**
     * approve teacher request engagement by teacher
     * @param $request
     * @return JsonResponse
     */
    public function approveEngagementRequestAction($request): JsonResponse
    {
        if ($this->getAuthUser()->role != 'teacher') {
            return response()->json([
                'message' => 'Sorry you are not authorized to access this route',
                'success' => false
            ], 403);
        }else {
            $engagement = $this->model->findOrFail($request->engagement_id);
            $class = $this->classesModel->findOrFail($request->class_id);
            try {
                $engagement->update([
                    'status' => true,
                    'class_id' => $request->class_id
                ]);
                if ($class->number_of_students === 10) {
                    return response()->json([
                        'message' => 'Sorry this class is already filled up',
                        'success' => false
                    ], 400);
                }
                $class->increment('number_of_students', 1);
                return response()->json([
                    'message' => 'Engagement approved successfully',
                    'data' => new UserResource($this->getAuthUser()),
                    'success' => true
                ], 200);
            }catch (\Exception $e) {
                return response()->json([
                    'message' => 'Sorry unable to approve engagement',
                    'error' => $e->getMessage(),
                    'success' => false
                ], 400);
            }
        }
    }

    /**
     * get meeting link
     * @param $id
     * @return ClassesResource|JsonResponse
     */
    public function getMeetingLinkAction($id): JsonResponse|ClassesResource
    {
        $engagement = $this->classesModel->findOrFail($id);
        $subject_name = $this->queryHelper->getSubjectName($engagement->subject_id);
        if (config('virtualclassroom-config.meeting.default') == 'zoom') {
            return (new ClassesResource($engagement))->additional( [
                'message' => "Class details",
                'success' => true
            ], 200);
        } else {
            $getUrl =  Http::asForm()->post($this->apiUrl."/v2/getclasslaunch?apikey=".$this->apiKey,  [
                'class_id' => $engagement->meeting_id,
                'userId' => $this->getAuthUser()->id,
                'userName' => $this->getAuthUser()->teacher ? $this->getAuthUser()->teacher->first_name :  $this->getAuthUser()->guardian->first_name,
                'isTeacher' => $this->getAuthUser()->role == 'teacher' ? 1 : 0,
                'lessonName'  =>  $subject_name. " ". "Lectures",
                'courseName'  =>  $subject_name. " ". "Lectures",
                'isExtend' => 0,
            ]);
            if(isset($getUrl['class_id'])) {
                return response()->json([
                    'message' => 'Class scheduled link',
                    'data' => $getUrl['launchurl'],
                    'success' => true
                ], 200);
            }
            return response()->json([
                'message' => $getUrl['error'],
                'success' => false
            ], 400);
        }
    }

    /**
     * get all teachers engagements
     * @return JsonResponse|AnonymousResourceCollection
     */
    public function getAllEngagementAction(): JsonResponse|AnonymousResourceCollection
    {
        $teachers_engagements = $this->getAuthUser()->teacher ?
            $this->model->where('teacher_id', '=', $this->getAuthUser()->teacher->id)->latest()->paginate(20)  :
            $this->model->where('guardian_id', '=', $this->getAuthUser()->guardian->id)->latest()->paginate(20);
        if (count($teachers_engagements) < 1) {
            return response()->json([
                'message' => 'Sorry no category found',
                'success' => false
            ], 404);
        }else {
            return TeacherRequestResource::collection($teachers_engagements)->additional([
                'message' => "All teachers engagements",
                'success' => true
            ], 200);
        }
    }

}
