<?php

namespace App\Libs\Actions;

use App\Http\Resources\UserResource;
use App\Libs\Enums\RolesEnum;
use App\Mail\AccountVerificationMail;
use App\Models\AccountVerificationToken;
use App\Models\User;
use Illuminate\Http\JsonResponse;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Mail;
use App\Libs\Helpers\SystemHelper;
use App\Libs\Traits\AuthUserTrait;

/**
 * user action
 */
class UserAction
{
    use AuthUserTrait;
    /**
     * @var User
     */
    private  User $model;
    /**
     * @var SystemHelper
     */
    private SystemHelper $system_helper;

    /**
     * @param User $model
     * @param SystemHelper $system_helper
     */
    public function __construct(User $model, SystemHelper $system_helper)
    {
        $this->model = $model;
        $this->system_helper = $system_helper;
    }

    /**
     * create merchant
     * @param $request
     * @return JsonResponse
     */
    public function createAccountAction($request): JsonResponse
    {
            DB::beginTransaction();
            try {
                if ($request->user_type == 'teacher') {
                    $user = $this->createUserHelper($request, RolesEnum::TEACHER);
                    $user->teacher()->create([
                        'first_name' => $request->first_name,
                        'last_name' => $request->last_name,
                        'phone_number' => $request->phone_number,
                    ]);
                } else {
                    $user = $this->createUserHelper($request, RolesEnum::GUARDIAN);
                    $user->guardian()->create([
                        'first_name' => $request->first_name,
                        'last_name' => $request->last_name,
                        'phone_number' => $request->phone_number,
                    ]);
                }
            } catch (\Exception $e) {
                DB::rollBack();
                return response()->json([
                    'message' => 'Sorry unable to create account',
                    'error' => $e->getMessage(),
                    'success' => false
                ], 400);
            }
            DB::commit();
            $this->mailWalkerForOnBoarding($request->email, $request->first_name);
            return response()->json([
                'message' => 'Account created successfully',
                'data' => new UserResource($this->model->find($user->id)),
                'success' => true
            ], 201);
    }

    /**
     * update profile
     * @param $request
     * @return JsonResponse
     */
    public function updateUserProfileAction($request): JsonResponse
    {
        if($this->getAuthUser()->guardian) {
            $this->updateEmail($request);
            $this->getAuthUser()->guardian()->update([
                'first_name' => empty($request->first_name) ? $this->getAuthUser()->guardian->first_name : $request->first_name,
                'last_name' => empty($request->last_name) ? $this->getAuthUser()->guardian->last_name : $request->last_name,
                'phone_number' => empty($request->last_name) ? $this->getAuthUser()->guardian->last_name : $request->last_name,
                'address' => empty($request->address) ? $this->getAuthUser()->guardian->address : $request->address
            ]);
            return response()->json([
                'message' => 'Account updated successfully',
                'data' => new UserResource($this->getAuthUser()),
                'success' => true
            ], 200);
        }elseif ($this->getAuthUser()->teacher) {
            $this->updateEmail($request);
            $this->getAuthUser()->teacher()->update([
                'first_name' => empty($request->first_name) ? $this->getAuthUser()->teacher->first_name : $request->first_name,
                'last_name' => empty($request->last_name) ? $this->getAuthUser()->teacher->last_name : $request->last_name,
                'school_attend' => empty($request->school_attend) ? $this->getAuthUser()->teacher->school_attend : $request->school_attend,
                'degree_attained' => empty($request->degree_attained) ? $this->getAuthUser()->teacher->degree_attained : $request->degree_attained,
                'phone_number' => empty($request->phone_number) ? $this->getAuthUser()->teacher->phone_number : $request->phone_number,
                'address' => empty($request->address) ? $this->getAuthUser()->teacher->address : $request->address,
                'bio' =>  empty($request->bio) ? $this->getAuthUser()->teacher->bio : $request->bio
            ]);
            return response()->json([
                'message' => 'Account updated successfully',
                'data' => new UserResource($this->getAuthUser()),
                'success' => true
            ], 200);
        }else {
            return response()->json([
                'message' => 'Sorry you are not allowed',
                'success' => false
            ], 405);
        }
    }

    /**
     * @param $request
     * @return void
     */
    public function updateEmail($request): void
    {
        $this->getAuthUser()->update([
            'email' => empty($request->email) ? $this->getAuthUser()->email : $request->email
        ]);
    }

    /**
     * create user helper
     * @param $request
     * @param $role
     * @return mixed
     */
    public function createUserHelper($request, $role): mixed
    {
        return $this->model->create([
            'email' => $request->email,
            'password' => bcrypt($request->password),
            'role' => $role
        ]);
    }

    /**
     * change user password
     * @param $request
     * @return JsonResponse
     */
    public function changePasswordAction($request): JsonResponse
    {
        $hashedPassword = $this->getAuthUser()->password;

        if (Hash::check($request->old_password , $hashedPassword)) {

            if (!Hash::check($request->password , $hashedPassword)) {

                try {
                    $this->getAuthUser()->update([
                        'password' => empty($request->password) ? $this->getAuthUser()->password : bcrypt($request->password),
                    ]);
                    return response()->json([
                        'message' => 'You have changed your password successfully',
                        'data' => new UserResource($this->getAuthUser()),
                        'success' => true
                    ], 200);

                }catch (\Exception $e) {
                    return response()->json([
                        'message' => 'Sorry the change password process failed',
                        'error' => $e->getMessage(),
                        'success' => false
                    ], 400);
                }

            }else {
                return response()->json([
                    'message' => 'Sorry new password can not be same as old password!',
                    'success' => false
                ], 401);
            }
        }else {
            return response()->json([
                'message' => 'Sorry old password doesnt matched',
                'success' => false
            ], 402);
        }
    }


    /**
     * resend otp to user email action
     * @param $request
     * @return JsonResponse
     */
    public function resendOtpCodeToEmailAction($request): JsonResponse
    {
        $user = $this->model->where('email', '=', $request->email)->firstOrFail();
        $this->mailWalkerForOnBoarding($request->email,  $request->email);
        return response()->json([
            'message' => 'Email sent successfully ',
            'data' => new UserResource($user),
            'success' => true
        ], 200);
    }

    /**
     * email walker
     * @param $email
     * @param $name
     */
    public function mailWalkerForOnBoarding($email, $name): void
    {
        $create =  AccountVerificationToken::create([
            'email' => $email,
            'token' => $this->system_helper->generateRandomOtp(6)
        ]);
        $email_user = new AccountVerificationMail([
            'otp' => $create->token,
            'name' => $name
        ]);
        Mail::to($email)->send($email_user);
    }

    /**
     * activate or deactivate user account
     * @param $request
     * @return JsonResponse
     */
    public function activateOrDeactivateUserAccountAction($request): JsonResponse
    {
        $user = $this->model->where('id', '=', $request->user_id)->firstOrFail();
        if ($request->status == 'approve') {
            $user->update([
                'is_active' => true
            ]);
            return response()->json([
                'message' => 'Account activated successfully',
                'data' => new UserResource($user),
                'success' => true
            ], 200);
        } else {
            $user->update([
                'is_active' => false
            ]);
            return response()->json([
                'message' => 'Account deactivate successfully',
                'data' => new UserResource($user),
                'success' => true
            ], 200);
        }
    }

}


