<?php

namespace App\Libs\Repositories;

use App\Http\Resources\TeacherResource;
use App\Libs\Repositories\Contracts\TeacherRepositoryInterface;
use App\Libs\Actions\TeacherAction;
use Illuminate\Http\JsonResponse;
use Illuminate\Http\Resources\Json\AnonymousResourceCollection;
use Illuminate\Support\Facades\Validator;

/**
 * Teachers repository
 */
class TeacherRepository implements TeacherRepositoryInterface
{
    /**
     * @var TeacherAction
     */
    private TeacherAction $action;

    /**
     * @param TeacherAction $action
     */
    public function __construct(TeacherAction $action) {
        $this->action = $action;
    }

    /**
     * add subject
     * @param $request
     * @return JsonResponse
     */
    public function addSubjects($request): JsonResponse
    {
        $validator = Validator::make($request->all(), [
            'subjects' => 'required|array',
            'category' => 'required|exists:categories,id'
        ], [
             'category.exists' => 'Please choose a category',
        ]);

        if ($validator->fails()) {
            return response()->json([
                'message' => $validator->messages()->first(),
                'success' => false
            ], 422);
        }else {
            return  $this->action->addSubjectsAction($request);
        }
    }

    /**
     * update teacher
     * @param $request
     * @return JsonResponse
     */
    public function updateEducationalQualification($request): JsonResponse
    {
        $validator = Validator::make($request->all(), [
            'school_attend' => 'required',
            'degree_attained' => 'required',
            'certificate_url' => 'required'
        ]);

        if ($validator->fails()) {
            return response()->json([
                'message' => $validator->messages()->first(),
                'success' => false
            ], 422);
        }else {
            return  $this->action->updateEducationalQualificationAction($request);
        }
    }

    /**
     * approve teacher
     * @param $slug
     * @return JsonResponse
     */
    public function approveTeachersAccount($slug): JsonResponse
    {
        return $this->action->approveTeachersAccountAction($slug);
    }

    /**
     * get teacher by subject
     * @param $request
     * @return JsonResponse|AnonymousResourceCollection
     */
    public function getTeachersBySubject($request): JsonResponse|AnonymousResourceCollection
    {
        $validator = Validator::make($request->all(), [
            'subject' => 'required'
        ]);

        if ($validator->fails()) {
            return response()->json([
                'message' => $validator->messages()->first(),
                'success' => false
            ], 422);
        }else {
            return $this->action->getTeachersBySubjectAction($request);
        }
    }

    /**
     * get single teacher
     * @param $id
     * @return TeacherResource
     */
    public function getSingleTeachers($id): TeacherResource
    {
        return $this->action->getSingleTeachersAction($id);
    }

    /**
     * get all teacher
     * @return JsonResponse|AnonymousResourceCollection
     */
    public function getAllTeachers(): JsonResponse|AnonymousResourceCollection
    {
        return $this->action->getAllTeachersAction();
    }
}
