<?php

namespace App\Libs\Repositories;

use App\Libs\Repositories\Contracts\UserRepositoryInterface;
use App\Libs\Actions\UserAction;
use Illuminate\Http\JsonResponse;
use Illuminate\Support\Facades\Validator;
use Illuminate\Validation\Rules\Password;

/**
 * user repository
 */
class UserRepository implements UserRepositoryInterface
{
    /**
     * @var UserAction
     */
    private  UserAction $action;

    /**
     * @param UserAction $action
     */
    public function __construct(UserAction $action)
    {
        $this->action = $action;
    }

    /**
     * create merchant
     * @param $request
     * @return JsonResponse
     */
    public function createUserAccount($request): JsonResponse
    {
        $validator = Validator::make($request->all(), [
            'first_name' => 'required',
            'last_name' => 'required',
            'email' => 'required|email|unique:users,email',
            'phone_number' => 'required',
            'user_type' => 'required',
            'password' => ['required', 'confirmed', Password::min(8)->letters()->mixedCase()->numbers()->symbols()]
        ]);

        if ($validator->fails()) {
            return response()->json([
                'message' => $validator->messages()->first(),
                'success' => false
            ], 422);
        }else {
            return $this->action->createAccountAction($request);
        }
    }

    /**
     * change password
     * @param $request
     * @return JsonResponse
     */
    public function changePassword($request): JsonResponse
    {
        $validator =  Validator::make($request->all(),[
            'old_password' => 'required',
            'password' => 'required|confirmed'
        ]);
        if ($validator->fails()) {
            return response()->json([
                'message' => $validator->errors()->first(),
                'success' => false
            ], 422);
        }else {
            return $this->action->changePasswordAction($request);
        }
    }

    /**
     * resend otp to user email
     * @param $request
     * @return JsonResponse
     */
    public function resendOtpCodeToEmail($request): JsonResponse
    {
        $validator =  Validator::make($request->all(),[
            'email' => 'required|email'
        ]);
        if ($validator->fails()) {
            return response()->json([
                'message' => $validator->errors()->first(),
                'success' => false
            ], 422);
        }else {
            return $this->action->resendOtpCodeToEmailAction($request);
        }
    }

    /**
     * activate or deactivate
     * @param $request
     * @return JsonResponse
     */
    public function activateOrDeactivate($request): JsonResponse
    {
        $validator =  Validator::make($request->all(),[
            'user_id' => 'required|exists:users,id',
            'status' => 'required'
        ]);
        if ($validator->fails()) {
            return response()->json([
                'message' => $validator->errors()->first(),
                'success' => false
            ], 422);
        }else {
           return $this->action->activateOrDeactivateUserAccountAction($request);
        }
    }

    /**
     * update profile
     * @param $request
     * @return JsonResponse
     */
    public function updateUserProfile($request): JsonResponse
    {
        $validator =  Validator::make($request->all(),[
            'first_name' => 'sometimes',
            'last_name' => 'sometimes',
            'email' => 'sometimes',
            'phone_number' => 'sometimes',
            'address' => 'sometimes',
            'bio' => 'sometimes'
        ]);
        if ($validator->fails()) {
            return response()->json([
                'message' => $validator->errors()->first(),
                'success' => false
            ], 422);
        }else {
            return  $this->action->updateUserProfileAction($request);
        }
    }

}
