<?php

namespace App\Libs\Services;

use App\Libs\Helpers\SystemHelper;
use App\Libs\Traits\AuthUserTrait;
use App\Models\Transaction;
use Illuminate\Http\JsonResponse;
use Yabacon\Paystack;

class PayStackService
{
    use AuthUserTrait;

    /**
     * @var Paystack
     */
    private Paystack $paystack;
    /**
     * @var Transaction
     */
    private Transaction $transaction_model;
    /**
     * @var SystemHelper
     */
    private SystemHelper $system_helper;

    /**
     * @param Transaction $transaction_model
     * @param SystemHelper $system_helper
     */
    public function __construct(Transaction $transaction_model, SystemHelper $system_helper)
    {
        $this->paystack = new Paystack(config('virtualclassroom-config.paystack.secret_key'));
        $this->transaction_model = $transaction_model;
        $this->system_helper = $system_helper;
    }

    /**
     * initialize stripe payment
     * @param $request
     * @return JsonResponse
     */
    public function initializePaystackPaymentService($request): JsonResponse
    {
        if ($this->getAuthUser()->guardian) {
            $tr = $this->createTransaction($request, 'paystack');
        try {
            $trx = $this->paystack->transaction->initialize(
                [
                    'amount' =>  $tr->amount, /* in kobo */
                    'email' => $this->getAuthUser()->email,
                    'reference' => $tr->trans_ref,
                    'callback_url' => "http://localhost:3000/dashboard/transactions?token=$tr->trans_ref",
                    'metadata' => [
                        'guardian_id' => $tr->guardian_id,
                        'reference' => $tr->trans_ref,
                        'transaction_id' => $tr->id,
                        'total' => $tr->amount,
                    ],
                ]
            );
            return response()->json([
                'message' => 'Paystack transaction link generated successfully',
                'url' => $trx->data->authorization_url,
                'success' => true
            ], 200);
        } catch (\Exception $e) {
            return response()->json([
                'message' => 'Sorry there was an error trying to generate payment link',
                'error' => $e->getMessage(),
                'success' => false
            ], 400);
        }
        }else {
            return response()->json([
                'message' => 'Sorry you are not allowed to carry out this operation',
                'success' => false
            ], 400);
        }
    }

    /**
     * verify paystack unit
     * @param $reference
     * @return JsonResponse
     */
    public function verifyPaystackPaymentService($reference): JsonResponse
    {
        if (!$reference) {
            return response()->json([
                'message' => 'Sorry No reference token supplied',
                'success' => false
            ], 404);
        }
        try {
            $trx = $this->paystack->transaction->verify([
                'reference' => $reference
            ]);
            if ('success' ===  $trx->data->status) {
                $transaction = $this->transaction_model->where('trans_ref', '=', $reference)->where('status', '=', false)->firstOrFail();
                $transaction->update([
                    'status' => true
                ]);
                if ($this->getAuthUser()->guardian) {
                    $this->getAuthUser()->guardian()->update([
                        'from_date' => now(),
                        'to_date' =>  $this->system_helper->subscriptionDurationHelper($transaction->plan_id),
                        'is_subscribed' => true,
                        'plan_id' => $transaction->plan_id
                    ]);
                } else {
                    return response()->json([
                        'message' => 'Sorry you are not allowed to carry out this operation',
                        'success' => false
                    ], 400);
                }
                return response()->json([
                    'message' => 'Transaction successful',
                    'success' => true
                ], 200);
            } else {
                return response()->json([
                    'message' => 'Sorry the transaction is unsuccessful',
                    'success' => true
                ], 400);
            }
        } catch (\Exception $e) {
            return response()->json([
                'message' => 'Sorry unable to verify transaction',
                'error' => $e->getMessage(),
                'success' => false
            ], 400);
        }
    }

    /**
     * create transaction record
     * @param $request
     * @param $gateway
     * @return mixed
     */
    public function createTransaction($request, $gateway): mixed
    {
        $paymentReference = "Trx" . sprintf("%0.9s", str_shuffle(rand(12, 30000) * time()));
        return $this->transaction_model->create([
            'guardian_id' => $this->getAuthUser()->guardian->id,
            'plan_id'  => $request->plan_id,
            'trans_ref' => $paymentReference,
            'amount' => $this->system_helper->convertToKobo($request->amount),
            'description' => $request->type,
            'payment_gateway' => $gateway,
            'type' => $request->type
        ]);
    }

}
