<?php

namespace Tests\Feature;

use App\Http\Controllers\Api\CategoryController;
use App\Libs\Actions\CategoryAction;
use App\Libs\Enums\RolesEnum;
use App\Libs\Repositories\CategoryRepository;
use App\Models\Category;
use Illuminate\Foundation\Testing\RefreshDatabase;;
use Illuminate\Support\Str;
use Laravel\Sanctum\Sanctum;
use Tests\TestCase;

class CategoryControllerTest extends TestCase
{
    use RefreshDatabase;

    public function test_admin_can_create_category()
    {
        $user = (new \App\Models\User)->create([
            'email' => fake()->safeEmail(),
            'email_verified_at' => now(),
            'password' => '$2y$10$92IXUNpkjO0rOQ5byMi.Ye4oKoEa3Ro9llC/.og/at2.uheWG/igi', // password
            'remember_token' => Str::random(10),
            'role' => RolesEnum::ADMIN
        ]);
        Sanctum::actingAs($user);

        $response = $this->postJson('/api/v1/categories', [
            "title" => "Web Development",
            "description" => "Web development classes"
        ]);
        $response->assertStatus(201);
    }

    public function test_other_user_type_cant_create_category()
    {
        $user = (new \App\Models\User)->create([
            'email' => fake()->safeEmail(),
            'email_verified_at' => now(),
            'password' => '$2y$10$92IXUNpkjO0rOQ5byMi.Ye4oKoEa3Ro9llC/.og/at2.uheWG/igi', // password
            'remember_token' => Str::random(10),
            'role' => RolesEnum::STUDENT
        ]);
        Sanctum::actingAs($user);
        $response = $this->postJson('/api/v1/categories', [
            "title" => "Web Development",
            "description" => "Web development classes"
        ]);
        $response->assertStatus(400);
    }

    public function test_validation_failed_on_category_create()
    {
        $user = (new \App\Models\User)->create([
            'email' => fake()->safeEmail(),
            'email_verified_at' => now(),
            'password' => '$2y$10$92IXUNpkjO0rOQ5byMi.Ye4oKoEa3Ro9llC/.og/at2.uheWG/igi', // password
            'remember_token' => Str::random(10),
            'role' => RolesEnum::ADMIN
        ]);
        Sanctum::actingAs($user);
        $response = $this->postJson('/api/v1/categories', [
            "title" => "",
            "description" => "Web development classes"
        ]);
        $response->assertStatus(422);
    }

    public function test_get_all_categories_no_category_found()
    {
        $user = (new \App\Models\User)->create([
            'email' => fake()->safeEmail(),
            'email_verified_at' => now(),
            'password' => '$2y$10$92IXUNpkjO0rOQ5byMi.Ye4oKoEa3Ro9llC/.og/at2.uheWG/igi', // password
            'remember_token' => Str::random(10),
            'role' => RolesEnum::ADMIN
        ]);
        Sanctum::actingAs($user);
        $response = $this->getJson('/api/v1/categories');
        $response->assertStatus(404);
    }

    public function test_get_all_categories()
    {
        $user = (new \App\Models\User)->create([
            'email' => fake()->safeEmail(),
            'email_verified_at' => now(),
            'password' => '$2y$10$92IXUNpkjO0rOQ5byMi.Ye4oKoEa3Ro9llC/.og/at2.uheWG/igi', // password
            'remember_token' => Str::random(10),
            'role' => RolesEnum::ADMIN
        ]);
        Sanctum::actingAs($user);
        $categories = Category::factory(10)->create();
        $response = $this->getJson('/api/v1/categories');
        $response->assertStatus(200);
    }

    public function test_get_single_category() {
        $user = (new \App\Models\User)->create([
            'email' => fake()->safeEmail(),
            'email_verified_at' => now(),
            'password' => '$2y$10$92IXUNpkjO0rOQ5byMi.Ye4oKoEa3Ro9llC/.og/at2.uheWG/igi', // password
            'remember_token' => Str::random(10),
            'role' => RolesEnum::ADMIN
        ]);
        Sanctum::actingAs($user);
        $categories = Category::factory()->create([
            "title" => "Web Development",
            "description" => "Web development classes"
        ]);
        $response = $this->getJson('/api/v1/categories/'.$categories->id);
        $response->assertStatus(200);
    }

    public function test_get_single_category_not_found()
    {
        $user = (new \App\Models\User)->create([
            'email' => fake()->safeEmail(),
            'email_verified_at' => now(),
            'password' => '$2y$10$92IXUNpkjO0rOQ5byMi.Ye4oKoEa3Ro9llC/.og/at2.uheWG/igi', // password
            'remember_token' => Str::random(10),
            'role' => RolesEnum::ADMIN
        ]);
        Sanctum::actingAs($user);
        $response = $this->getJson('/api/v1/categories/2');
        $response->assertStatus(404);
    }

    public function test_delete_category_successful()
    {
        $user = (new \App\Models\User)->create([
            'email' => fake()->safeEmail(),
            'email_verified_at' => now(),
            'password' => '$2y$10$92IXUNpkjO0rOQ5byMi.Ye4oKoEa3Ro9llC/.og/at2.uheWG/igi', // password
            'remember_token' => Str::random(10),
            'role' => RolesEnum::ADMIN
        ]);
        Sanctum::actingAs($user);
        $category = Category::factory()->create([
            "title" => "Web Development",
            "description" => "Web development classes"
        ]);
        $response = $this->deleteJson('/api/v1/categories/'.$category->id);
        $response->assertStatus(200);
    }

    public function test_delete_category_filed()
    {
        $user = (new \App\Models\User)->create([
            'email' => fake()->safeEmail(),
            'email_verified_at' => now(),
            'password' => '$2y$10$92IXUNpkjO0rOQ5byMi.Ye4oKoEa3Ro9llC/.og/at2.uheWG/igi', // password
            'remember_token' => Str::random(10),
            'role' => RolesEnum::ADMIN
        ]);
        Sanctum::actingAs($user);
        $response = $this->deleteJson('/api/v1/categories/2');
        $response->assertStatus(404);
    }

    public function test_update_category_successful()
    {
        $user = (new \App\Models\User)->create([
            'email' => fake()->safeEmail(),
            'email_verified_at' => now(),
            'password' => '$2y$10$92IXUNpkjO0rOQ5byMi.Ye4oKoEa3Ro9llC/.og/at2.uheWG/igi', // password
            'remember_token' => Str::random(10),
            'role' => RolesEnum::ADMIN
        ]);
        Sanctum::actingAs($user);
        $category = Category::factory()->create([
            "title" => "Web Development",
            "description" => "Web development classes"
        ]);
        $response = $this->deleteJson('/api/v1/categories/'.$category->id, [
            "title" => "Web Development Updated",
            "description" => "Web development classes updated"
        ]);
        $response->assertStatus(200);
    }

    public function test_update_category_failed()
    {
        $user = (new \App\Models\User)->create([
            'email' => fake()->safeEmail(),
            'email_verified_at' => now(),
            'password' => '$2y$10$92IXUNpkjO0rOQ5byMi.Ye4oKoEa3Ro9llC/.og/at2.uheWG/igi', // password
            'remember_token' => Str::random(10),
            'role' => RolesEnum::ADMIN
        ]);
        Sanctum::actingAs($user);
        $response = $this->deleteJson('/api/v1/categories/2', [
            "title" => "Web Development Updated",
            "description" => "Web development classes updated"
        ]);
        $response->assertStatus(404);
    }

    public function test_class_has_attributes()
    {
        $this->assertClassHasAttribute('categoryRepositoryInterface', CategoryController::class);
        $this->assertClassHasAttribute('action', CategoryRepository::class);
        $this->assertClassHasAttribute('model', CategoryAction::class);
    }

}
